package be.ac.ulb.code.wit;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.namespace.QName;
import javax.xml.ws.handler.Handler;
import javax.xml.ws.handler.HandlerResolver;
import javax.xml.ws.handler.MessageContext;
import javax.xml.ws.handler.PortInfo;

/**
 * Utility class to manage chains of custom handlers for a given service
 * 
 * @param <T> The type of message contexts to be handled.
 */
public class MyHandlerResolver<T extends MessageContext> implements HandlerResolver {
	Map<String, List<Handler<T>>> handlers = new HashMap<String, List<Handler<T>>>();
	List<Handler<T>> globalHandlers = new ArrayList<Handler<T>>();
	
	/**
	 * Register an handler for a specific service.
	 * @param serviceLocalName The name of the service for which the handler must be registered
	 * @param handler The handler to be called for messages specific to the service
	 */
	public void registerServiceHandler(String serviceLocalName, Handler<T> handler) {
		if (!handlers.containsKey(serviceLocalName)) {
			handlers.put(serviceLocalName, new ArrayList<Handler<T>>());
		}
		
		handlers.get(serviceLocalName).add(handler);
	}
	
	/**
	 * Register a handler for all services.
	 * @param handler The handler to be registered.
	 */
	public void registerHandler(Handler<T> handler) {
		globalHandlers.add(handler);
	}

	/**
	 * Provide handler for the specified ports, according to handlers registered previously.
	 */
	@SuppressWarnings("rawtypes")
	public List<Handler> getHandlerChain(PortInfo portInfo) {
       List<Handler> handlerChain = new ArrayList<Handler>();

       QName serviceQName = portInfo.getServiceName();
       
       if (handlers.containsKey(serviceQName.getLocalPart())) {
    	   handlerChain.addAll(handlers.get(serviceQName.getLocalPart()));
       }
       handlerChain.addAll(globalHandlers);
       
       return handlerChain;
	}


}
